
;**************************************************************************
;*                             Count Down Switch                          *
;**************************************************************************
;*pokec*
;*pokec*
;**************************************************************************
;*          v 0.05 - 17.05.2009                                           *
;**************************************************************************
;*          Hardw.: PIC16F628A used                                       *
;*          OSC.......: Internal osc. used     POWER.....:  5,0V 7805     *
;**************************************************************************

W                            EQU     H'0000'
F                            EQU     H'0001'

;----- Register Files------------------------------------------------------

INDF                         EQU     H'0000'
TMR0                         EQU     H'0001'
PCL                          EQU     H'0002'
STATUS                       EQU     H'0003'
FSR                          EQU     H'0004'
PORTA                        EQU     H'0005'
PORTB                        EQU     H'0006'

INTCON                       EQU     H'000B'
OPTION_REG                   EQU     H'0081'
TRISA                        EQU     H'0085'
TRISB                        EQU     H'0086'
CMCON                        EQU     H'001F'

;----- STATUS Bits --------------------------------------------------------
IRP                          EQU     H'0007'
RP1                          EQU     H'0006'
RP0                          EQU     H'0005'
NOT_TO                       EQU     H'0004'
NOT_PD                       EQU     H'0003'
Z                            EQU     H'0002'
DC                           EQU     H'0001'
C                            EQU     H'0000'

;==========================================================================
;
;       RAM Definition
;
;==========================================================================

    __MAXRAM H'01FF'
    __BADRAM H'07'-H'09', H'0D', H'13'-H'14', H'1B'-H'1E'
    __BADRAM H'87'-H'89', H'8D', H'8F'-H'91', H'93'-H'97', H'9E'
    __BADRAM H'105', H'107'-H'109', H'10C'-H'11F', H'150'-H'16F'
    __BADRAM H'185', H'187'-H'189', H'18C'-H'1EF'

;==========================================================================
;
;       Configuration Bits
;
;==========================================================================

_BODEN_ON                    EQU     H'3FFF'
_BODEN_OFF                   EQU     H'3FBF'
_CP_ALL                      EQU     H'03FF'
_CP_75                       EQU     H'17FF'
_CP_50                       EQU     H'2BFF'
_CP_OFF                      EQU     H'3FFF'
_DATA_CP_ON                  EQU     H'3EFF'
_DATA_CP_OFF                 EQU     H'3FFF'
_PWRTE_OFF                   EQU     H'3FFF'
_PWRTE_ON                    EQU     H'3FF7'
_WDT_ON                      EQU     H'3FFF'
_WDT_OFF                     EQU     H'3FFB'
_LVP_ON                      EQU     H'3FFF'
_LVP_OFF                     EQU     H'3F7F'
_MCLRE_ON                    EQU     H'3FFF'
_MCLRE_OFF                   EQU     H'3FDF'
_ER_OSC_CLKOUT               EQU     H'3FFF'
_ER_OSC_NOCLKOUT             EQU     H'3FFE'
_INTRC_OSC_CLKOUT            EQU     H'3FFD'
_INTRC_OSC_NOCLKOUT          EQU     H'3FFC'
_EXTCLK_OSC                  EQU     H'3FEF'
_LP_OSC                      EQU     H'3FEC'
_XT_OSC                      EQU     H'3FED'
_HS_OSC                      EQU     H'3FEE'

	__CONFIG        _BODEN_OFF & _CP_OFF & _DATA_CP_OFF & _PWRTE_ON & _WDT_OFF & _LVP_OFF & _MCLRE_ON & _INTRC_OSC_NOCLKOUT

;==========================================================================
;       Variable Definition
;==========================================================================
;INPUTS
SW1		EQU     H'00'		;SW1 is triggering RA0
SW2		EQU     H'01'		;SW2 is triggering RA1
SW3		EQU     H'02'		;SW3 is triggering RA2
SW4		EQU     H'03'		;SW4 is triggering RA3
TIMER1		EQU	H'20'		;Used in delay routine
TIMER2		EQU	H'21'		; "	"	"	
timer3		EQU	H'22'
stage10		EQU	H'23'
stage10b	EQU	H'24'
;==========================================================================

		ORG	0		;Reset vector address
		GOTO	RESET		;goto RESET routine when boot.


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


;	       **********************************
;              **  RESET :  main boot routine  **
;              **********************************

RESET		MOVLW	B'00000111'	;Disable Comparator module's
		MOVWF	CMCON
		;
		BSF	STATUS,RP0	;Switch to register bank 1
					;Disable pull-ups
					;INT on rising edge
					;TMR0 to CLKOUT
					;TMR0 Incr low2high trans.
					;Prescaler assign to Timer0
					;Prescaler rate is 1:256
		MOVLW	B'11010111'	;Set PIC options (See datasheet).
		MOVWF	OPTION_REG	;Write the OPTION register.
		;
		CLRF	INTCON		;Disable interrupts
		MOVLW	B'00000000'
		MOVWF	TRISB		;RBx
		MOVLW	B'00110010'	;RA4, RA5 & RA1 - input
		MOVWF	TRISA
		BCF	STATUS,RP0	;Switch Back to reg. Bank 0

; XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
; XXXXXXXXXX       main            XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
; XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

main		bsf	PORTA,D'000'		;switch power on

		movlw	B'11111111'
		movwf	PORTB
		bsf	PORTA,D'006'
		bsf	PORTA,D'007'

;.....
;----------------------------------------------------------------
;stage00
		movlw	D'010'
		movwf	stage10

stg00		bsf	PORTA,D'007'
		bsf	PORTA,D'006'
		movlw	D'255'
		movwf	PORTB
		CALL	dr3
		clrf	PORTB
		bcf	PORTA,D'007'
		bcf	PORTA,D'006'
		CALL	dr3
		DECFSZ	stage10,F
		GOTO	stg00
;----------------------------------------------------------------
		movlw	B'11111111'
		movwf	PORTB
		bsf	PORTA,D'006'
		bsf	PORTA,D'007'

		goto	stg_3s		;after reset - default = 8 hrs

;----------------------------------------------------------------
;stage01
		movlw	D'020'
		movwf	stage10b
stg01b		movlw	D'180'
		movwf	stage10
stg01		bsf	PORTB,D'007'		;1st green led on
		CALL	dr2
		bcf	PORTB,D'007'		;1st green led off
		CALL	dr2
		DECFSZ	stage10,F
		GOTO	stg01
		DECFSZ	stage10b,F
		GOTO	stg01b
;----------------------------------------------------------------
;stage02
		movlw	D'020'
		movwf	stage10b
stg02b		movlw	D'180'
		movwf	stage10
stg02		bsf	PORTB,D'006'		;2nd green led on
		CALL	dr2
		bcf	PORTB,D'006'		;2nd green led off
		CALL	dr2
		DECFSZ	stage10,F
		GOTO	stg02
		DECFSZ	stage10b,F
		GOTO	stg02b
;----------------------------------------------------------------
;stage03
stg_3s		bcf	PORTB,D'007'
		bcf	PORTB,D'006'
		movlw	D'020'
		movwf	stage10b
stg03b		movlw	D'180'
		movwf	stage10
stg03		bsf	PORTB,D'005'		;3rd green led on
		CALL	dr2
		bcf	PORTB,D'005'		;3rd green led off
		CALL	dr2
		DECFSZ	stage10,F
		GOTO	stg03
		DECFSZ	stage10b,F
		GOTO	stg03b
;----------------------------------------------------------------
;stage04
		movlw	D'020'
		movwf	stage10b
stg04b		movlw	D'180'
		movwf	stage10
stg04		bsf	PORTB,D'004'		;4th green led on
		CALL	dr2
		bcf	PORTB,D'004'		;4th green led off
		CALL	dr2
		DECFSZ	stage10,F
		GOTO	stg04
		DECFSZ	stage10b,F
		GOTO	stg04b
;----------------------------------------------------------------
;stage05
		movlw	D'020'
		movwf	stage10b
stg05b		movlw	D'180'
		movwf	stage10
stg05		bsf	PORTB,D'003'		;5th green led on
		CALL	dr2
		bcf	PORTB,D'003'		;5th green led off
		CALL	dr2
		DECFSZ	stage10,F
		GOTO	stg05
		DECFSZ	stage10b,F
		GOTO	stg05b
;----------------------------------------------------------------
;stage06
		movlw	D'020'
		movwf	stage10b
stg06b		movlw	D'180'
		movwf	stage10
stg06		bsf	PORTB,D'002'		;6th green led on
		CALL	dr2
		bcf	PORTB,D'002'		;6th green led off
		CALL	dr2
		DECFSZ	stage10,F
		GOTO	stg06
		DECFSZ	stage10b,F
		GOTO	stg06b
;----------------------------------------------------------------
;stage07
		movlw	D'020'
		movwf	stage10b
stg07b		movlw	D'180'
		movwf	stage10
stg07		bsf	PORTB,D'001'		;last green led on
		CALL	dr2
		bcf	PORTB,D'001'		;last green led off
		CALL	dr2
		DECFSZ	stage10,F
		GOTO	stg07
		DECFSZ	stage10b,F
		GOTO	stg07b
;----------------------------------------------------------------
;stage08
		movlw	D'020'
		movwf	stage10b
stg08b		movlw	D'180'
		movwf	stage10
stg08		bsf	PORTB,D'000'		;first red led on
		CALL	dr2
		bcf	PORTB,D'000'		;first red led off
		CALL	dr2
		DECFSZ	stage10,F
		GOTO	stg08
		DECFSZ	stage10b,F
		GOTO	stg08b
;----------------------------------------------------------------
;stage09
		movlw	D'020'
		movwf	stage10b
stg09b		movlw	D'180'
		movwf	stage10
stg09		bsf	PORTA,D'007'		;2nd red led on
		CALL	dr2
		bcf	PORTA,D'007'		;2nd red led off
		CALL	dr2
		DECFSZ	stage10,F
		GOTO	stg09
		DECFSZ	stage10b,F
		GOTO	stg09b
;----------------------------------------------------------------
;stage10
		movlw	D'020'
		movwf	stage10b
stg10b		movlw	D'180'
		movwf	stage10
stg10		bsf	PORTA,D'006'		;last red led on
		CALL	dr2
		bcf	PORTA,D'006'		;last red led off
		CALL	dr2
		DECFSZ	stage10,F
		GOTO	stg10
		DECFSZ	stage10b,F
		GOTO	stg10b
;----------------------------------------------------------------
;final_stage
stg_fg		clrf	PORTB

		movlw	D'020'
		movwf	stage10
stg11		bsf	PORTA,D'006'
		bsf	PORTA,D'007'
		bsf	PORTB,D'000'
		CALL	dr3
		bcf	PORTA,D'006'
		bcf	PORTA,D'007'
		bcf	PORTB,D'000'
		CALL	dr3
		DECFSZ	stage10,F
		GOTO	stg11
;-----------------------------------------------------------------
;switch off
		clrf	PORTA			;switch off...

end_of_story	goto	end_of_story
;----------------------------------------------------------
;		*********************************************
;		*               delay routine               *
;		*********************************************

;0,5sec delay at 4MHz clock

DELAY_ROUTINE	movlw	D'050'
DELAY_ROUTINE2	movwf	timer3
;10ms delay
DEL_LOOP0	MOVLW   D'054'
		MOVWF   TIMER2
DEL_LOOP1       MOVLW   D'060'
		MOVWF   TIMER1
DEL_LOOP2	DECFSZ  TIMER1,F
		GOTO    DEL_LOOP2

		btfss	PORTA,D'001'
		goto	stg_fg

		DECFSZ  TIMER2,F
		GOTO    DEL_LOOP1
		DECFSZ  timer3,F
		GOTO    DEL_LOOP0
		return
;-------------------------------------------
dr2		call	DELAY_ROUTINE
		return
;-------------------------------------------
dr3		movlw	D'030'
		call	DELAY_ROUTINE2
		return
;-----------------------------------------------------------
		END